@@ -1,32 +1,35 @@
-use strict
-require 5.004;
+use strict;
+use warnings;
+require 5.6.0;
 package Parse::Template;
-$Parse::Template::VERSION = '0.34';
+$Parse::Template::VERSION = '3.00';
 
-use constant DEBUG => 0;	
-use constant AUTOLOAD_TRACE => 0;
+use Carp;
+use constant DEBUG => 0;
 use vars qw/$AUTOLOAD/;
 sub AUTOLOAD {
   my($class, $part) = ($AUTOLOAD =~ /(.*)::(.*)$/);
-  print STDERR "AUTOLOAD=$AUTOLOAD\n\tclass=$class\n\tpart=$part\n\targs=@_\n" if AUTOLOAD_TRACE;
-  *$AUTOLOAD = sub { 
-    (ref $_[0] || $class)->eval("$part", @_) 
-  };
+  no strict 'refs';
+  *$AUTOLOAD = sub { (ref $_[0] || $class)->eval("$part", @_) };
   goto &$AUTOLOAD;
 }
 
+use Symbol qw(delete_package);
 { my $id = 0; sub getid { $id++ } }
 
 my $PACKAGE = __PACKAGE__;
 sub new {
   my $receiver = shift;
   my $class = $PACKAGE . '::Sym' . getid();
-  my $self = bless {}, $class;	# no info in $self
-  no strict;
-  @{"${class}::ISA"} = (ref $receiver || $receiver);
-  %{"${class}::template"} = ();	
-  %{"${class}::template"} = @_ ;
+  my $self = bless {}, $class;	# absolutely nothing in $self
+  no strict 'refs';
+  %{"${class}::template"} = ();	# so no 'used only once' warning
+  ${"${class}::ancestor"} = '';	# so no 'used only once' warning
+
+  @{"${class}::ISA"} = ref $receiver || $receiver;
+  ${"${class}::ancestor"} = $receiver;	# reverse the destruction order
   *{"${class}::AUTOLOAD"} = \&AUTOLOAD; # so no warning for procedural calls
+  %{"${class}::template"} = @_ ;
   $self;
 }
 use constant TRACE_ENV => 0;
@@ -35,44 +38,45 @@ sub env {
   my $class = ref $self || $self;
   my $symbol = shift;
   if ($symbol =~ /\W/) {
-    require Carp;
     Carp::croak "invalid symbol name: $symbol"
   }
-
   no strict;
   if (@_) {
-    while (@_) {
+    do {
       my $value = shift;
       print STDERR "${class}::$symbol\t$value\n" if TRACE_ENV;
       if (ref $value) {
-	*{"${class}::$symbol"} = $value;
+		*{"${class}::$symbol"} = $value;
       } else {			# scalar value
       	*{"${class}::$symbol"} = \$value;
       }
       $symbol = shift if @_;
       if ($symbol =~ /\W/) {
-	require Carp;
-	Carp::croak "invalid symbol name: $symbol"
+		Carp::croak "invalid symbol name: $symbol";
       }
-    }
-  } elsif (defined *{"${class}::$symbol"}) { # borrowed from Exporter.pm
+    } while (@_);
+  } 
+  elsif (defined *{"${class}::$symbol"}) { # borrowed from Exporter.pm
     return \&{"${class}::$symbol"} unless $symbol =~ s/^(\W)//;
     my $type = $1;
     return 
-      $type eq '*' ?  *{"${class}::$symbol"} :
-	$type eq "\$" ? \${"${class}::$symbol"} :
-	  $type eq '%' ? \%{"${class}::$symbol"} :
-	    $type eq '@' ? \@{"${class}::$symbol"} :
-	      $type eq '&' ? \&{"${class}::$symbol"} :
-		do { require Carp; Carp::croak("Can\'t find symbol: $type$symbol") };
-  } else {
+		$type eq '*' ?  *{"${class}::$symbol"} :
+		$type eq "\$" ? \${"${class}::$symbol"} :
+		$type eq '%' ? \%{"${class}::$symbol"} :
+		$type eq '@' ? \@{"${class}::$symbol"} :
+		$type eq '&' ? \&{"${class}::$symbol"} :
+		do { Carp::croak("Can\'t find symbol: $type$symbol") };
+  } 
+  else {
     undef;
   }
 }
 sub DESTROY {
-  print STDERR "destroy @_\n" if DEBUG;
+  print STDERR "destroy(@_): ", ref $_[0], "\n" if DEBUG;
+  delete_package(ref $_[0]);
 }
-# Purpose:  validate the regexp and replace "!" by "\!", and "/" by "\/"
+# Purpose: validate the regexp and replace "!" by "\!", and "/" by "\/" 
+#          if not already escaped
 # Arguments: a regexp
 # Returns: the preprocessed regexp
 sub ppregexp {
@@ -80,8 +84,7 @@ sub ppregexp {
   my $regexp = $_[1];
   eval { '' =~ /$regexp/ };
   if ($@) {	
-    $@ =~ s/\s+at\s+[^\s]+\s+line\s+\d+[.]\n$//; # annoying...
-    require Carp;
+    $@ =~ s/\s+at\s+[^\s]+\s+line\s+\d+[.]\n$//; # annoying info
     Carp::croak $@;	
   }
   $regexp =~ s{
@@ -99,7 +102,6 @@ sub getPart {
   unless (defined($text = ${"${class}::template"}{$part})) {
      my $parent = ${"${class}::ISA"}[0]; # delegation
      unless (defined $parent) {
-      require Carp;
       Carp::croak("'$part' template part is not defined");
     }
     $text = $parent->getPart($part);
@@ -113,22 +115,22 @@ sub setPart {
   no strict 'refs';
   ${"${class}::template"}{$part} = shift; 
 }
-$Parse::Template::CONFESS = 1; 
+$Parse::Template::CONFESS = 1;
 my $Already_shown = 0;
 my $__DIE__ = sub { 
   if  (not($Parse::Template::CONFESS) and $Already_shown) {
     # Reset when the eval() processing is finished
     $Already_shown = 0 if defined($^S); 
     return;
-  } 
-
-  # evaluated expressions are not always available in (caller(1))[6];	
+  }
+  # evaluated expressions are not always available in (caller(1))[6];
   if (defined($1) and $1 ne '') {
-    $expr = $1;			# what is the template expression?
+    my $expr = $1;		# what is the template expression?
     { package DB;		# what is the part name?
-      @DB::caller = caller(2);	# why is this needed? /ee?
       @DB::caller = caller(1);
+      @DB::caller = caller(2) unless @DB::args;
     };	
+    #local $1;
     $expr =~ s/package\s+${PACKAGE}::\w+\s*;//o;
     my $line = 0;
     $expr =~ s/^/sprintf "%2s ", ++$line/egm;
@@ -139,29 +141,32 @@ my $__DIE__ = sub {
     } else {
       print STDERR "Error in part '$part':\n$expr\n";
     }
+  } 
+  else {
+    print STDERR "\$1 not defined";    
   }
+  print STDERR "\$1: $1\n";    
   # ignore Already_shown if you won't confess your exception
   $Already_shown = 1 unless $Parse::Template::CONFESS;
 };
-#$SIG{__WARN__} = sub {
-#  require Carp;
-#Carp::confess; # cant properly die???
-#Carp::cluck;
-#};
+$Parse::Template::SIG{__WARN__} = sub { # don't know how to suppress this:
+  print STDERR "$_[0]" 
+    unless ($_[0] =~ /^Use of uninitialized value in substitution iterator/)
+};
+
 use constant EVAL_TRACE => 0;
 use constant SHOW_PART => 0;
 use constant SIGN_PART => 0;
 $Parse::Template::SIGN_START = "# Template %s {\n"; # not documented
 $Parse::Template::SIGN_END = "# } Template %s\n"; # not documented
 my $indent = 0;
-my $part = '';			# a Perl bug? - workaround!!!
 my @part = ();
 sub eval {
   print STDERR do { 
     local $" = q!', '! ; '..' x ++$indent, "=>eval('@_')\n" 
   } if EVAL_TRACE;
   my $self = shift;
-  $part = shift;		# can't declare $part in eval()
+  my $part = shift;		# can't declare $part in eval()
   push @part, $part;
   my $class = ref $self || $self;
   my $text = $self->getPart($part);
@@ -171,11 +176,13 @@ sub eval {
     $text =~ s!$!sprintf $Parse::Template::SIGN_END, $part!e;
   }
   local $SIG{__DIE__} = $__DIE__;
-  $text =~ s( %% (.*?) %% ){	# the magic substitution
-    print STDERR "Part name: $part\n" if EVAL_TRACE;
+  # eval expression in class
+  $text =~ s( %% (.*?) %% ){	# the magical substitution
+    print STDERR '..' x $indent, "Eval part name: $part\n" if EVAL_TRACE;
     print STDERR '..' x $indent, "  expr: package $class;\n$1\n" if EVAL_TRACE;
     "package $class; $1";
   }eegsx;
+  print STDERR "after: $class - $1\n" if EVAL_TRACE;
   die "$@" if $@;		# caught by __DIE__
   pop @part; $part = $part[-1];
   --$indent if EVAL_TRACE;
@@ -186,31 +193,31 @@ __END__
 
 =head1 NAME
 
-Parse::Template - Processor for templates containing Perl expressions 
+Parse::Template - Processor for templates containing Perl expressions
 
 =head1 SYNOPSIS
 
   use Parse::Template;
 
-  my %template =
+  my %template = 
     (
      'TOP' =>  q!Text before %%$self->eval('DATA')%% text after!,
      'DATA' => q!Insert data: ! .
                q!1. List: %%"@list$N"%%! .
-               q!2. Hash: %%"$hash{'key_value'}$N"%%! .
+               q!2. Hash: %%"$hash{'key'}$N"%%! .
                q!3. File content: %%print <FH>%%! .
                q!4. Sub: %%&SUB()$N%%!
     );
-
+ 
   my $tmplt = new Parse::Template (%template);
   open FH, "< foo";
 
   $tmplt->env('var' => '(value!)');
-  $tmplt->env('list' => [1, 2, 10],
+  $tmplt->env('list' => [1, 2, 10], 
               'N' => "\n",
               'FH' => \*FH,
               'SUB' => sub { "->content generated by a sub<-" },
-              'hash' => { 'key_value' => q!It\'s an hash value! });
+              'hash' => { 'key' => q!It\'s an hash value! });
   print $tmplt->eval('TOP'), "\n";
 
 =head1 DESCRIPTION
@@ -224,7 +231,7 @@ The principle of template-based text generation is simple.  A template
 consists of a text which includes expressions to be evaluated.
 Interpretation of these expressions generates text fragments which are
 substituted in place of the expressions.  In the case of
-C<Parse::Template> the expressions to be evaluated are placed within
+C<Parse::Template> the expressions to be evaluated are Perl expressions placed within
 two C<%%>.
 
 Evaluation takes place within an environment in which, for example,
@@ -244,7 +251,7 @@ The C<Parse::Template> class permits decomposing a template into
 parts.  These parts are defined by a hash passed as an argument to the
 class constructor:
 C<Parse::Template->E<gt>C<new('someKey', '... text with expressions to
-evaluate ...')>.  Within a part, a sub-part can beincluded by means of
+evaluate ...')>.  Within a part, a sub-part can be included by means of
 an expression of the form:
 
   $self->eval('SUB_PART_NAME')
@@ -287,8 +294,10 @@ that contains the reference.
 
 Each instance of C<Parse::Template> is defined within a specific class,
 a subclass of C<Parse::Template>.  The subclass contains the environment
-specific to the template and inherits methods from the C<Parse::Template>
-class.
+specific to the template and inherits methods from the C<Parse::Template> class.
+
+If a template is created from an existing template (i.e. calling
+C<new> as a method of the existing template), it inherits all the parts defined by its ancestor.
 
 In case of a syntax error in the evalutaion of an expression,
 C<Parse::Template> tries to indicate the template part and the
@@ -307,8 +316,8 @@ template text.
 
 Example:
 
- use Parse::Template;
- $t = new Parse::Template('key' => 'associated text');
+  use Parse::Template;
+  $t = new Parse::Template('key' => 'associated text');
 
 =item env HASH
 
@@ -354,121 +363,165 @@ contents of the template.
 
 =head1 EXAMPLES
 
-
 The C<Parse::Template> class can be used in all sorts of amusing
 ways. Here are a few illustrations.
 
+=head2 HTML Generator
+
 The first example shows how to generate an HTML document by using a
 data structure placed within the evaluation environment.  The template
 consists of two parts, C<DOC> and C<SECTION>.  The C<SECTION> part is
 called within the C<DOC> part to generate as many sections as there are
 elements in the array C<section_content>.
 
- my %template = ('DOC' => <<'END_OF_DOC;', 'SECTION' => <<'END_OF_SECTION;');
- <html>
- <head></head>
- <body>
- %%
- my $content;
- for (my $i = 0; $i <= $#section_content; $i++) {
-   $content .= SECTION($i);
- }
- $content;
- %%
- </body>
- </html>
- END_OF_DOC;
- %%
- $section_content[$_[0]]->{Content} =~ s/^/<p>/mg;
- join '', '<H1>', $section_content[$_[0]]->{Title}, '</H1>', $section_content[$_[0]]->{Content};
- %%
- END_OF_SECTION;
+  my %template = ('DOC' => <<'END_OF_DOC;', 'SECTION' => <<'END_OF_SECTION;');
+  <html>
+  <head></head>
+  <body>
+  %%
+  my $content;
+  for (my $i = 0; $i <= $#section_content; $i++) {
+    $content .= SECTION($i);
+  }
+  $content;
+  %%
+  </body>
+  </html>
+  END_OF_DOC;
+  %%
+  $section_content[$_[0]]->{Content} =~ s/^/<p>/mg;
+  join '', '<H1>', $section_content[$_[0]]->{Title}, '</H1>',
+                   $section_content[$_[0]]->{Content};
+  %%
+  END_OF_SECTION;
 
- my $tmplt = new Parse::Template (%template);
+  my $tmplt = new Parse::Template (%template);
 
- $tmplt->env('section_content' => [
-     {
-      Title => 'First Section',
-      Content => 'Nothing to declare'
-     },
-     {
-      Title => 'Second section',
-      Content => 'Nothing else to declare'
-     }
+  $tmplt->env('section_content' => [
+      {
+        Title => 'First Section',
+        Content => 'Nothing to write'
+      },
+      {
+        Title => 'Second section',
+        Content => 'Nothing else to write'
+      }
     ]
-     );
+  );
 
- print $tmplt->eval('DOC'), "\n";
+  print $tmplt->eval('DOC'), "\n";
+
+=head2 HTML generation using functional notation
 
 The second example shows how to generate an HTML document using a
 functional notation, in other words, obtaining the text:
 
- <P><B>text in bold</B><I>text in italic</I></P>
+  <P><B>text in bold</B><I>text in italic</I></P>
 
-from
+from:
 
- P(B("text in bold"), I("text in italic"))
+  P(B("text in bold"), I("text in italic"))
 
-
-The Perl expression that permits producing the content of an element is
+The functions P(), B() and I() are defined as parts of a template.  The Perl expression
+that permits producing the content of an element is
 very simple, and reduces to:
 
- join '', @_
+  join '', @_
 
 The content to be evaluated is the same regardless of the tag and can
 therefore be placed within a variable.  We therefore obtain the
 following template:
 
- my $ELT_CONTENT = q!%%join '', @_%%!;
- my $HTML_T1 = new Parse::Template(
+  my $ELT_CONTENT = q!%%join '', @_%%!;
+  my $HTML_T1 = new Parse::Template(
        'DOC' => '%%P(B("text in bold"), I("text in italic"))%%',
-       'P' => qq!<P>$ELT_CONTENT</P>!,
-       'B' => qq!<B>$ELT_CONTENT</B>!,
-       'I' => qq!<I>$ELT_CONTENT</I>!,
+       'P'   => qq!<P>$ELT_CONTENT</P>!,
+       'B'   => qq!<B>$ELT_CONTENT</B>!,
+       'I'   => qq!<I>$ELT_CONTENT</I>!,
       );
- print $HTML_T1->eval('DOC'), "\n";
+  print $HTML_T1->eval('DOC'), "\n";
 
 We can go further by making use of the C<$part> variable, which
 is defined by default in the environment of evaluation of the template:
 
- $ELT_CONTENT = q!%%"<$part>" . join('', @_) . "</$part>"%%!;
- $HTML_T2 = new Parse::Template(
+  my $ELT_CONTENT = q!%%"<$part>" . join('', @_) . "</$part>"%%!;
+  my $HTML_T2 = new Parse::Template(
        'DOC' => '%%P(B("text in bold"), I("text in italic"))%%',
-       'P' => qq!$ELT_CONTENT!,
-       'B' => qq!$ELT_CONTENT!,
-       'I' => qq!$ELT_CONTENT!,
+       'P'   => qq!$ELT_CONTENT!,
+       'B'   => qq!$ELT_CONTENT!,
+       'I'   => qq!$ELT_CONTENT!,
       );
- print $HTML_T2->eval('DOC'), "\n";
-
+  print $HTML_T2->eval('DOC'), "\n";
 
 Let's look at another step which automates the production of 
 expressions from the list of HTML tags which are of interest to us:
 
- $DOC = q!P(B("text in bold"), I("text in italic"))!;
+  my $DOC = q!P(B("text in bold"), I("text in italic"))!;
+  my $ELT_CONTENT = q!%%"<$part>" . join('', @_) . "</$part>"%%!;
+  my $HTML_T3 = new Parse::Template(
+       'DOC' => qq!%%$DOC%%!,
+       map { $_ => $ELT_CONTENT } qw(P B I)
+      );
+  print $HTML_T3->eval('DOC'), "\n";
 
- $ELT_CONTENT = q!%%"<$part>" . join('', @_) . "</$part>"%%!;
- $HTML_T3 = new Parse::Template(
-      'DOC' => qq!%%$DOC%%!,
-      map { $_ => $ELT_CONTENT } qw(P B I)
-     );
- print $HTML_T3->eval('DOC'), "\n";
+To benefit from the possibility of using the template parts as procedures, we can
+inherit from the generated template class:
 
+  use Parse::Template;
+  my $ELT_CONTENT = q!%%"<$part>" . join('', @_) . "</$part>"%%!;
+  my $G = new Parse::Template(
+       map { $_ => $ELT_CONTENT } qw(H1 B I)
+      );
+  @main::ISA = ref($G);
+  *AUTOLOAD = \&Parse::Template::AUTOLOAD;
+  print H1(B("text in bold"), I("text in italic"));
+
+The reference to C<Parse::Template::AUTOLOAD> avoids the warning message:
+
+  Use of inherited AUTOLOAD for non-method %s() is deprecated
+
+Not very elegant.
+
+=head2 HTML generation by method call
 
 With a slight transformation it is possible to use a 
 method-invocation notation:
 
- $ELT_CONTENT = q!%%shift(@_); "<$part>" . join('', @_) . "</$part>"%%!;
- $HTML_T4 = new Parse::Template(
-      map { $_ => $ELT_CONTENT } qw(P B I)
-     );
- print $HTML_T4->P(
-                   $HTML_T4->B("text in bold"),
-                   $HTML_T4->I("text in italic")
-                  ), "\n";
+  my $ELT_CONTENT = q!%%shift(@_); "<$part>" . join('', @_) . "</$part>"%%!;
+  my $HTML_T4 = new Parse::Template(
+       map { $_ => $ELT_CONTENT } qw(P B I)
+      );
+  print $HTML_T4->P(
+                    $HTML_T4->B("text in bold"),
+                    $HTML_T4->I("text in italic")
+                   ), "\n";
 
 The C<shift(@_)> permits getting rid of the template object, which
 we don't need within the expression.
 
+=head2 Inheritance of parts
+
+In the following example the child template C<$C> inherits the parts defined 
+in its parent template C<$A>:
+
+  my %ancestor = 
+    (
+     'TOP' => q!%%"Use the $part model and -> " . CHILD()%%!,
+     'ANCESTOR' => q!ANCESTOR %%"'$part' part\n"%%!,
+    );
+
+  my %child = 
+    (
+     'CHILD' => q!CHILD %%"'$part' part"%% -> %%ANCESTOR() . "\n"%%!,
+    );
+  my $A = new Parse::Template (%ancestor);
+  my $C = $A->new(%child);
+  print $C->TOP();
+
+The part <TOP> defined in C<$A> can be called directly from C<$C>, that derives from C<$A>.
+
+=head2 Other examples
+
 C<Parse::Template> was initially created to serve as a code generator
 for the C<Parse::Lex> class.  You will find other examples of its use
 in the classes C<Parse::Lex>, C<Parse::CLex> and C<Parse::Token>.
@@ -476,20 +529,19 @@ in the classes C<Parse::Lex>, C<Parse::CLex> and C<Parse::Token>.
 =head1 NOTES CONCERNING THE CURRENT VERSION
 
 I would be very interested to receive your comments and suggestions.
-English documentation isn't up to date.
 
-=head1 BUG
+=head1 BUGS
 
 Instances are not destroyed.  Therefore, do not use this class 
 to create a large number of instances.
 
 =head1 AUTHOR
 
-Philippe Verdret (with translation of documentation into English by Ocrat).
+Philippe Verdret (with translation of documentation into English by Ocrat)
 
 =head1 COPYRIGHT
 
-Copyright (c) 1995-2000 Philippe Verdret. All rights reserved.
+Copyright (c) 1995-2001 Philippe Verdret. All rights reserved.
 This module is free software; you can redistribute it and/or
 modify it under the same terms as Perl itself.
 
